// 全局配置对象
const globalConfig = {
    xiezuocatUrl: 'https://xiezuocat.com',
    xiezuocatAPIUrl: 'https://xiezuocat.com',
    ocrAPIUrl: "http://ocr_api_mt.metaso.cn/api/single_img?crop&&format_type=1",
    // logToEsBaseUrl: 'https://metasotalaw.cn',
    logToEsBaseUrl: 'https://metaso.cn',
    // uranusUrl:'https://metasotalaw.cn',
    uranusUrl:'https://metaso.cn',
    uploadUrl: "https://service.metapro.cn/metacloud/remote.php/webdav/",
    projectName: 'xiezuocat-Chromium-browser-extension',
    //客服端类型(1:chrome浏览器插件)
    clientType: 1,
    versionNo: '1.1.8',
    isLogin: false,
    normalNotice: null,
    //重要的通知
    importNotice: null,
    //版本更新
    versionUpdate: null,
    uid: null,
    sid: null,
    user: null,
    product: null,
    //插件黑名单
    blackList: [],
    //用户当前网站功能开关
    userBlackList: [],
    //是否是调试模式
    isDebug: false,
    onLineTim: '',

};
chrome.browserAction.onClicked.addListener(function (tab) {

});

const LOGIN = "LOGIN";
const LOGIN_UPDATE = "LOGIN_UPDATE";
const LOGOUT = "LOGOUT";
const VERSION = "version";


setIsLogin(getGlobalConfig().isLogin, function () {}, false)

//初始化函数
function init() {
    let config = getGlobalConfig();
    if (!config.isDebug) {
        //获取cookie里面的uid和sid，并在都获取到之后获取用户信息。
        refreshUidAndSid();
        refreshBlackList();
        refreshUserBlackList();
        setOnLineTime();
    }
}

function addVersionInfoToRequestHeader(originHeader) {
    originHeader.append('xiezuocat-ce-version', getGlobalConfig().versionNo);
    return originHeader;
}

init();

function updateBrowserAction() {
    let config = getGlobalConfig();
    if (config.versionUpdate) {
        // 如果当前有新版本
        chrome.browserAction.setBadgeText({text: 'new'});
        chrome.browserAction.setBadgeBackgroundColor({color: [255, 0, 0, 255]});
    } else {
        // 如果当前没有新版本
        chrome.tabs.query({active: true, currentWindow: true}, function (tabs) {
            let domain = getUrlDomain(tabs[0].url);
            let isExist = config.userBlackList.some(function (filteredDomain) {
                return filteredDomain === domain;
            })
            if (isExist) {
                chrome.browserAction.setBadgeText({text: 'off'});
                chrome.browserAction.setBadgeBackgroundColor({color: [255, 0, 0, 255]});
            } else {
                chrome.browserAction.setBadgeText({text: ''});
                chrome.browserAction.setBadgeBackgroundColor({color: [255, 0, 0, 255]});
            }
        });
    }

}

function getLoginStatus() {
    // return isLogin
    return getGlobalConfig().isLogin
}

/**
 * 刷新用户自定义黑名单
 */
function refreshUserBlackList() {
    let config = getGlobalConfig();
    if (config.isLogin && config.uid) {
        let userBlackListParms = {};
        userBlackListParms['userBlackLis' + config.uid] = [];
        chrome.storage.sync.get(userBlackListParms, function (item) {
            config.userBlackList = item['userBlackLis' + config.uid];
        });
    } else {
        config.userBlackList = [];
    }
}

function addUserBlackList(domain) {
    let config = getGlobalConfig();
    if (config.isLogin && config.uid) {
        let isExist = config.userBlackList.some(function (filteredDomain) {
            return filteredDomain === domain;
        })
        if (!isExist) {
            config.userBlackList.push(domain);
            let userBlackListParms = {};
            userBlackListParms['userBlackLis' + config.uid] = config.userBlackList;
            chrome.storage.sync.set(userBlackListParms, function () {
                updateBrowserAction();
            });
            //发送广播到当前Tab所有已初始化的checker插件中
            broadcastMessageToActiveTabContentScript({cmd: 'addUserBlackList', data: null});
        }
    } else {
        config.userBlackList = [];
    }
}

function deleteUserBlackList(domain) {
    let config = getGlobalConfig();
    if (config.isLogin && config.uid) {
        let newUserBlackList = [];
        for (let i = 0; i < config.userBlackList.length; i++) {
            if (config.userBlackList[i] !== domain) {
                newUserBlackList.push(config.userBlackList[i]);
            }
        }
        config.userBlackList = newUserBlackList;
        let userBlackListParms = {};
        userBlackListParms['userBlackLis' + config.uid] = config.userBlackList;
        chrome.storage.sync.set(userBlackListParms, function () {

            updateBrowserAction();
        });
    } else {
        config.userBlackList = [];
    }
}

chrome.tabs.onActivated.addListener(function (activeInfo) {
    updateBrowserAction();
});


//更新标签页的时候刷新用户信息(新建标签也会触发这个事件，所以不监听onCreated事件)
chrome.tabs.onUpdated.addListener(function (id, info, tab) {
    if (info.status === 'complete') {
        refreshUidAndSid();
    }
});

var isActiveNoviceGuide = false;
chrome.storage.sync.get({isActiveNoviceGuide: isActiveNoviceGuide}, function (item) {
    isActiveNoviceGuide = item.isActiveNoviceGuide;
});


chrome.runtime.onSuspend.addListener(function () {
});


/* Listen for external messages (messages from web-pages) */
chrome.runtime.onMessageExternal.addListener(function (msg, sender) {

    if (sender.url.startsWith(getGlobalConfig().xiezuocatUrl)) {
        /* OK, this page is allowed to communicate with me */
        if (msg.status === LOGIN) {
            refreshUidAndSid()
        } else if (msg.status === LOGOUT) {
            setIsLogin(false, function () {
            })
        } else if (msg.status === LOGIN_UPDATE) {
            refreshUidAndSid()
        }
    }
});


chrome.runtime.onMessageExternal.addListener(function (request, sender, sendResponse) {
    if (request) {
        if (request.message) {
            if (request.message === VERSION) {
                sendResponse({version: "1.1.8"})
            }
        }
    }
    return true;
});

chrome.runtime.onMessage.addListener(function (request, sender, sendResponse) {
    if (request.contentScriptQuery === "checkContent") {
        let myHeaders = new Headers();
        myHeaders.append('Content-Type', 'application/json');
        myHeaders = addVersionInfoToRequestHeader(myHeaders);
        let req = new Request(getGlobalConfig().xiezuocatAPIUrl + "/html/checkContent", {
            method: "POST",
            headers: myHeaders,
            mode: "cors",
            credentials: "include",
            body: JSON.stringify(request.actualParams),
        })
        fetch(req)
        // .then(response => response.json())
            .then(function (response) {
                return response.text().then(function (text) {
                    sendResponse([{
                        body: text,
                        status: response.status,
                        statusText: response.statusText,
                    }, null]);
                });
            }, function (error) {
                sendResponse([null, error]);
            });
        return true;  // Will respond asynchronously.
    }
    if ("getGlobalConfig" === request.cmd) {
        sendResponse(getGlobalConfig());
    } else if ("refreshUserInfo" === request.cmd) {
        refreshUidAndSid()
    } else if ("refreshBlackList" === request.cmd) {
        let config = getGlobalConfig();
        sendResponse(config.blackList);
        if (config.blackList.length <= 0) {
            refreshBlackList();
        }
    } else if ("getUserBlackList" === request.cmd) {
        let config = getGlobalConfig();
        sendResponse(config.userBlackList);
    } else if ("logToEs" === request.cmd) {
        logToES(request.data)
    } else if ("logOnLineTimeToEs" === request.cmd) {
        logOnLineTime();
    } else if ("ocrRequest" === request.cmd) {
        let myHeaders = new Headers();
        myHeaders.append('Content-Type', 'application/json');
        myHeaders = addVersionInfoToRequestHeader(myHeaders);
        let req = new Request(getGlobalConfig().ocrAPIUrl, {
            method: "POST",
            headers: myHeaders,
            mode: "cors",
            credentials: "include",
            body: JSON.stringify(request.actualParams),
        });
        fetch(req)
            .then(function (response) {
                return response.text().then(function (text) {
                    sendResponse([{
                        body: text,
                        status: response.status,
                        statusText: response.statusText,
                    }, null]);
                });
            }, function (error) {
                sendResponse([null, error]);
            });
        return true;  // Will respond asynchronously.
    }else if ("ocrResultCopy" === request.cmd) {
        let copyDiv = document.createElement('div');
        copyDiv.contentEditable = true;
        copyDiv.style = "white-space:pre-wrap;background: rgb(255,255,255,0);"
        document.body.appendChild(copyDiv);
        copyDiv.innerHTML = request.data;
        copyDiv.unselectable = 'off';
        copyDiv.focus();
        document.execCommand('SelectAll');
        document.execCommand('Copy', false, null);
        document.body.removeChild(copyDiv);
        sendResponse({
            copyStatus: true
        });
    }
});

/**
 * parseHtml
 */
function parseHtml(htmlText,successFunc) {
    let config = getGlobalConfig();
    let myHeaders = new Headers();
    myHeaders.append('Content-Type', 'application/json');
    myHeaders = addVersionInfoToRequestHeader(myHeaders);
    let req = new Request(config.uranusUrl+ "/extract_html", {
        method: "POST",
        headers: myHeaders,
        mode: "cors",
        credentials: "include",
        body: JSON.stringify({text:htmlText})
    })
    fetch(req).then((response) => {
        if (response.status === 200) {
            response.json().then(function(json){
                successFunc(json)
            });
            return {};
        } else {
            throw Error(`Request rejected with status ${response.status}`);
        }
    }).then((res) => {
        config.blackList = res;
    }).catch(err => {
        config.blackList = [];
    });
}


/**
 * 发送获取黑名单信息请求
 */
function refreshBlackList() {
    let config = getGlobalConfig();
    let myHeaders = new Headers();
    myHeaders.append('Content-Type', 'application/json');
    myHeaders = addVersionInfoToRequestHeader(myHeaders);
    let req = new Request(getGlobalConfig().xiezuocatAPIUrl + "/html/getWebPluginBlackList", {
        method: "GET",
        headers: myHeaders,
        mode: "cors",
        credentials: "include",
    })
    fetch(req).then((response) => {
        if (response.status === 200) {
            return response.json();
        } else {
            throw Error(`Request rejected with status ${response.status}`);
        }
    }).then((res) => {
        config.blackList = res;
    }).catch(err => {
        config.blackList = [];
    });
}

/**
 * 设置是否登录
 * @param isLogin
 * @param successCallback
 * @param listenChange
 */
function setIsLogin(isLogin, successCallback, listenChange = true) {
    chrome.storage.sync.set({isLogin: isLogin}, function () {
        // isLogin = true;
        let config = getGlobalConfig();
        if (isLogin) {
            loginSuccess();
        } else {
            config.userBlackList = [];
            setUser(null);
            setProduct(null);

        }
        config.isLogin = isLogin;
        if (successCallback) {
            successCallback()
        }
    });
}

/**
 * 设置普通通知信息
 */
function setNormalNotice(normalNotice) {
    getGlobalConfig().normalNotice = normalNotice;
    let popupView = chrome.extension.getViews({type: 'popup'});
    if (popupView.length > 0) {
        popupView[0].popupSetNormalNotice(normalNotice);
    }
}

/**
 * 设置重要通知类型
 */
function setImportantNotice(importantNotice) {
    getGlobalConfig().importNotice = importantNotice;
    //发送广播到所有已初始化的checker插件中
    broadcastMessageToContentScript({cmd: 'importantNotice', data: importantNotice})
}

/**
 * 设置版本更新信息
 */
function setVersionUpdateInfo(versionUpdate) {
    //更新版本更新信息到储存中
    getGlobalConfig().versionUpdate = versionUpdate;
    //发送广播到所有已初始化的checker插件中
    broadcastMessageToContentScript({cmd: 'versionUpdate', data: versionUpdate});
    updateBrowserAction();
    let popupView = chrome.extension.getViews({type: 'popup'});
    if (popupView.length > 0) {
        popupView[0].popupSetVersionUpdate(versionUpdate);
    }
}

/**
 * 获取配置。
 * @return {{xiezuocatUrl: string, xiezuocatAPIUrl: string, clientType: number, versionNo: string, isLogin: boolean, normalNotice: null, importNotice: null, versionUpdate: null, uid: null, sid: null, user: null, product: null, isDebug: boolean,uranusUrl}}
 */
function getGlobalConfig() {
    return globalConfig
}

/**
 * 登录成功的回调
 */
function loginSuccess() {
    //获取最新的通知。
    getNewestNotice().then(res => {
        res.text().then(function (text) {
            let resJSON = JSON.parse(text);
            if (!resJSON) {
                setNormalNotice(null);
                setImportantNotice(null);
                return
            }
            let noticeInfo = JSON.parse(resJSON.errMsg);
            if (!noticeInfo) {
                setNormalNotice(null);
                setImportantNotice(null);
                return
            }
            setNormalNotice(noticeInfo.normalNotice);
            setImportantNotice(noticeInfo.importantNotice);
        });
    }).catch(err => {
        console.error(err);
    })
    getVersionUpdate().then(res => {
        res.text().then(function (text) {
            let resJSON = JSON.parse(text);
            if (!resJSON) {
                return
            }
            let newestVersion = JSON.parse(resJSON.errMsg);
            if (!newestVersion) {
                setVersionUpdateInfo(null);
                return;
            }
            if (getGlobalConfig().versionNo !== newestVersion.versionNo) {
                setVersionUpdateInfo(newestVersion);
            } else {
                setVersionUpdateInfo(null);
            }
        });
    }).catch(err => {
        console.error(err);
    })
    refreshUserBlackList();
}

function getUrlDomain(url) {
    //切分
    var domain = url.split('/');
    if (domain[2]) {
        domain = domain[2];
    } else {
        domain = ''; //如果url不正确就取空
    }
    return domain;
}

/**
 * 发送消息给所有tab的contentScript
 * @param message 消息
 * @param callback （每一个tab处理完毕后的回调）
 */
function broadcastMessageToContentScript(message, callback) {
    chrome.tabs.query({}, function (tabs) {
        for (let i in tabs) {
            chrome.tabs.sendMessage(tabs[i].id, message, function (response) {
                if (callback) callback(response);
            });
        }
    });
}

/**
 * 发送消息给当前tab的contentScript
 * @param message 消息
 * @param callback （每一个tab处理完毕后的回调）
 */
function broadcastMessageToActiveTabContentScript(message, callback) {
    chrome.tabs.query({active: true, currentWindow: true}, function (tabs) {
        for (let i in tabs) {
            chrome.tabs.sendMessage(tabs[i].id, message, function (response) {
                if (callback) callback(response);
            });
        }
    });
}

/**
 * 获取最新公告
 */
function getNewestNotice() {
    let myHeaders = new Headers();
    myHeaders = addVersionInfoToRequestHeader(myHeaders);
    let req = new Request(`${getGlobalConfig().xiezuocatAPIUrl}/html/getNewestNotice?clientType=${getGlobalConfig().clientType}`, {
        method: "GET",
        headers: myHeaders,
        mode: "cors",
        credentials: "include"
    })
    return fetch(req)
}

/**
 * 获取版本更新
 */
function getVersionUpdate() {
    let myHeaders = new Headers();
    myHeaders = addVersionInfoToRequestHeader(myHeaders);
    let req = new Request(`${getGlobalConfig().xiezuocatAPIUrl}/html/getNewestVersion?clientType=${getGlobalConfig().clientType}`, {
        method: "GET",
        headers: myHeaders,
        mode: "cors",
        credentials: "include"
    })
    return fetch(req)
}

//设置uid
function setUid(uid) {
    getGlobalConfig().uid = uid;
}

//设置sid
function setSid(sid) {
    getGlobalConfig().sid = sid;
}

function setUser(user) {
    getGlobalConfig().user = user;
}

function setProduct(product) {
    getGlobalConfig().product = product
}

//如果uid和sid齐全，就获取用户信息
function getUserInfoIfNeed() {
    let config = getGlobalConfig();
    if (config.uid && config.sid) {
        refreshUserInfo()
    }
}

function refreshUidAndSid() {
    getCookies(this.getGlobalConfig().xiezuocatAPIUrl, "uid", function (res) {
        if (!res) {
            //说明未登录
            setIsLogin(false);
            return;
        }
        setUid(res.value);
    })
    getCookies(this.getGlobalConfig().xiezuocatAPIUrl, "sid", function (res) {
        if (!res) {
            //说明未登录
            setIsLogin(false);
            return;
        }
        setSid(res.value);
    })
    refreshUserInfo();
}

//发送获取用户信息请求
function refreshUserInfo() {
    let config = getGlobalConfig();
    if (!config.uid || !config.sid) {
        setIsLogin(false);
        return;
    }
    let myHeaders = new Headers();
    myHeaders = addVersionInfoToRequestHeader(myHeaders);
    let req = new Request(`${getGlobalConfig().xiezuocatAPIUrl}/html/myInfo?uid=${config.uid}&sid=${config.sid}`, {
        method: "GET",
        headers: myHeaders,
        mode: "cors",
        credentials: "include"
    });
    fetch(req).then(function (response) {
        response.text().then(function (text) {
            let resJSON = filterResponseText(text);
            if (!resJSON) {
                return
            }
            let userInfo = JSON.parse(resJSON.msg);
            if (!userInfo) {
                return
            }

            setUser(userInfo.user);
            setProduct(userInfo.product);
            setIsLogin(true)
            // refreshUserProduct(function () {
            //     if (getUserSuccessFunc) {
            //         getUserSuccessFunc(userInfo);
            //     }
            // }, function () {
            //     if (getUserFailedFunc) {
            //         getUserFailedFunc();
            //     }
            // })
        });
    }, function (error) {
        setIsLogin(false)
    });
}

// function refreshUserProduct(successFunc, failedFunc) {
//     let config = getGlobalConfig();
//     if (!config.user) {
//         return
//     }
//     let req = new Request(`${getGlobalConfig().xiezuocatAPIUrl}/html/getUserProduct?productType=${config.user.productType}`, {
//         method: "GET",
//         headers: new Headers(),
//         mode: "cors",
//         credentials: "include"
//     });
//     fetch(req).then(function (response) {
//         response.text().then(function (text) {
//             let product = JSON.parse(text);
//             if (!product) {
//                 return
//             }
//             setProduct(product);
//             if (successFunc) {
//                 successFunc(product)
//             }
//         });
//     }, function (error) {
//         if (failedFunc) {
//             failedFunc(error);
//         }
//     });
// }

function filterResponseText(text) {
    let resJSON = JSON.parse(text);
    if (!resJSON) {
        setIsLogin(false);
        return null;
    }
    if (resJSON.errCode === -100 || resJSON.errCode === 401 || resJSON.code === -100 || resJSON.code === 401) {
        setIsLogin(false)
        return null;
    }
    return resJSON;
}


function setCookies(url, name, value, expireSecond) {
    //var exdate = new Date();
    var param = {
        url: url,
        name: name,
        value: value,
        path: '/'
    };
    if (!!expireSecond) {
        param.expirationDate = new Date().getTime() / 1000 + expireSecond;
    }
    chrome.cookies.set(param, function (cookie) {
    });
}

function getCookies(url, key, callback, isAutoDelay, params) {
    chrome.cookies.get({
        url: url,
        name: key
    }, function (cookies) {
        if (params) {
            callback(cookies, params);
        } else {
            callback(cookies);
        }
    });
}

function setOnLineTime() {
    let config = getGlobalConfig();
    let nowTime = new Date().getTime()
    config.onLineTime = nowTime.toString();
}


function logOnLineTime() {
    let lastOnLineTime = getGlobalConfig().onLineTime;
    let nowTime = new Date().getTime();
    let updateOnLineTime = true;
    if (lastOnLineTime.length > 0) {
        let interval_time = nowTime - parseInt(lastOnLineTime);
        if (interval_time < 60000) {
            //间隔时间没有超过一分钟，认为用户在线,更新时间差,且为了防止过于频繁触发事件更新，规定间隔事件必须大于1S才进行时间差更新
            if (interval_time > 1000) {
                _logToES({event_name: 'browserExtensionOnLineTime', value: interval_time});
            } else {
                updateOnLineTime = false;
            }
        } else {
            //间隔时间超过一分钟，认为用户不在线，从30s开始计时
            _logToES({event_name: 'browserExtensionOnLineTime', value: 30000});
        }
    } else {
        //第一次记录
    }
    if (updateOnLineTime) {
        setOnLineTime();
    }
}

function logToES(_params) {
    logOnLineTime();
    let params = _params;
    _logToES(params);
}

/**
 * 埋点，数据存到es
 * @params {event_name, project_name, value} 参数必填
 */
function _logToES(_params) {
    getCookies(this.getGlobalConfig().xiezuocatAPIUrl, "traceid", function (res) {
        try {
            let traceid = null;
            if (res && res.value) {
                traceid = res.value;
            }
            let config = getGlobalConfig();
            let params = _params;
            let now = new Date();
            let UTCNow = new Date();
            UTCNow.setFullYear(now.getUTCFullYear());
            UTCNow.setMonth(now.getUTCMonth());
            UTCNow.setDate(now.getUTCDate());
            UTCNow.setHours(now.getUTCHours());
            UTCNow.setMinutes(now.getUTCMinutes());
            UTCNow.setSeconds(now.getUTCSeconds());
            let user_id = "";
            if (config.isLogin && config.uid) {
                user_id = config.uid;
            }

            let browserType = getBrowserType();
            params['project_name'] = config.projectName;
            params['traceid'] = traceid == null ? "" : traceid;
            params['user_id'] = user_id;
            params['timestamp'] = Math.floor(UTCNow / 1000);
            params['browser_type'] = browserType == null ? "" : browserType;
            let myHeaders = new Headers();
            myHeaders.append('Content-Type', 'application/json');
            myHeaders = addVersionInfoToRequestHeader(myHeaders);
            let req = new Request(config.logToEsBaseUrl + "/admin/user/event_report", {
                method: "POST",
                headers: myHeaders,
                mode: "cors",
                credentials: "include",
                body: JSON.stringify(params),
            })
            fetch(req).then(res => {
            }).catch(err => {
            })
        } catch (err) {
            console.log('logToES  err:', err)
        }
    })
}

function getBrowserType() {
    let ua = navigator.userAgent.toLocaleLowerCase();
    let browserType = null;
    if (ua.match(/msie/) != null || ua.match(/trident/) != null) {
        browserType = "IE";
    } else if (ua.match(/firefox/) != null) {
        browserType = "firefox";
    } else if (ua.match(/ubrowser/) != null) {
        browserType = "UC";
    } else if (ua.match(/opera/) != null) {
        browserType = "opera";
    } else if (ua.match(/bidubrowser/) != null) {
        browserType = "baidu";
    } else if (ua.match(/metasr/) != null) {
        browserType = "sogo";
    } else if (ua.match(/tencenttraveler/) != null || ua.match(/qqbrowse/) != null) {
        browserType = "QQ";
    } else if (ua.match(/maxthon/) != null) {
        browserType = "aoyou";
    } else if (ua.match(/edge/) != null || ua.match(/edg/) != null) {
        browserType = "edge";
    } else if (ua.match(/chrome/) != null) {
        var is360 = _mime("type", "application/vnd.chromium.remoting-viewer");

        function _mime(option, value) {
            var mimeTypes = navigator.mimeTypes;
            for (var mt in mimeTypes) {
                if (mimeTypes[mt][option] == value) {
                    return true;
                }
            }
            return false;
        }

        if (is360) {
            browserType = '360';
        } else {
            browserType = 'google';
        }
    } else if (ua.match(/safari/) != null) {
        browserType = "Safari";
    }
    return browserType;
}

function uploadHtml(req_data) {

    let fullHtml = "<html>\n" +
        "<head>\n" +
        "    <meta charset=\"UTF-8\"/>\n" +
        "    <title>" + req_data.title + "</title>\n" +
        "</head>\n" +
        req_data.html +
        "</html>";
    let XHR = new XMLHttpRequest();

    let conf = getGlobalConfig();
    try {
        XHR.open("PUT", conf.uploadUrl + req_data.title + ".html", false);
        XHR.setRequestHeader("Content-Type", "text/html");
        XHR.setRequestHeader("meta-uid", conf.uid);
        XHR.setRequestHeader("meta-sid", conf.sid);
        XHR.setRequestHeader("meta-request", "auth");
        XHR.send(fullHtml);
    } catch (e) {
        return false;
    }
    return true;

}


function uploadPdf(req_data) {

    const pdf_url = req_data.url;
    var leftIndex = pdf_url.lastIndexOf("/");
    var rightIndex = pdf_url.lastIndexOf(".pdf");
    var fileName = pdf_url.slice(leftIndex + 1, rightIndex + 4);

    const myRequest = new Request(pdf_url, {method: 'GET'});
    fetch(myRequest).then(response => response.arrayBuffer()).then(body => {
        const blob = new Blob([body], {type: 'application/pdf'});
        let XHR = new XMLHttpRequest();
        let conf = getGlobalConfig();
        XHR.open("PUT", conf.uploadUrl + fileName, false);
        XHR.setRequestHeader("Content-Type", "application/pdf");
        XHR.setRequestHeader("meta-uid", conf.uid);
        XHR.setRequestHeader("meta-sid", conf.sid);
        XHR.setRequestHeader("meta-request", "auth");
        XHR.send(blob);

    }).catch(error => {
        console.error(error);
        return false;
    });
    return true;
}


chrome.extension.onMessage.addListener(
    //监听扩展程序进程或内容脚本发送请求的请求
    function (request, sender, sendResponse) {
        let action = request.action;

        if (action === "save_pdf") {
            request.req_data["contentType"] = "PDF";
            let result = uploadPdf(request.req_data);
            sendResponse({result: result})
        } else if (action === "save_main_content") {
            request.req_data["contentType"] = "HTML";
            let result = uploadHtml(request.req_data);
            sendResponse({result: result})
        } else if (action === 'capture-screen') {
            chrome.tabs.captureVisibleTab(function (dataURL) {
                chrome.tabs.query({active: true, currentWindow: true}, function (tabs) {
                    for (let i in tabs) {
                        chrome.tabs.getZoom(tabs[i].id, function (zf) {
                            sendResponse({
                                dataURL: dataURL,
                                zf: zf
                            });
                        });
                    }
                });

            });
            return true;
        }
    }
);
