/*global $, chrome*/
$(function () {
    let background = chrome.extension.getBackgroundPage();
    initLogin(background);
    let domain = null;
    chrome.tabs.getSelected(null, function (tab) {
        domain = background.getUrlDomain(tab.url);
        if (domain.indexOf("xiezuocat.com") >= 0 || domain.indexOf("checker.metasotalaw.cn") >= 0) {
            $('#action_page').hide();
            $('#checker_page').show();
        } else {
            initUserBlackList(background, domain);
            $('#action_page').show();
            $('#checker_page').hide();
        }
    });
    initNormalNotice(background);
    initVersionUpdate(background);
    background.logToES({event_name: 'popupOpenButtonClick', text: '', value: 1});
});

/**
 * 根据登录状态初始化页面
 * @param background
 */
function initLogin(background) {
    var islogin = background.getLoginStatus();
    if (islogin) {
        $('#popup-warp-not-login').hide();
        $('#popup-warp-login').show();
    } else {
        $('#popup-warp-not-login').show();
        $('#popup-warp-login').hide();
    }
}

/**
 * 初始化普通通知
 * @param background
 */
function initNormalNotice(background) {
    let normalNotice = background.getGlobalConfig().normalNotice
    let normalNoticeJItem = $(".popup-meta-checker-normalNotice");
    if (normalNotice) {
        normalNoticeJItem.attr('href', normalNotice.url);
        normalNoticeJItem.text(normalNotice.title);
        normalNoticeJItem.show()
    } else {
        normalNoticeJItem.hide()
    }
}

/**
 * 初始化版本更新
 * @param background
 */
function initVersionUpdate(background) {
    let versionUpdate = background.getGlobalConfig().versionUpdate
    let versionUpdateJItem = $(".popup-meta-checker-versionUpdate")
    if (versionUpdate) {
        versionUpdateJItem.attr('href', versionUpdate.url);
        versionUpdateJItem.text(versionUpdate.title);
        versionUpdateJItem.show()
    } else {
        versionUpdateJItem.hide()
    }
}

function initUserBlackList(background, domain) {
    let config = background.getGlobalConfig();
    if (config.isLogin) {
        $('#popup-meta-checker-function-switch-host').text(domain);
        let isInBlackList = false;
        for (let i = 0; i < config.userBlackList.length; i++) {
            if (config.userBlackList[i] === domain) {
                isInBlackList = true;
                break;
            }
        }
        if (!isInBlackList) {
            $("#popup-meta-checker-function-switch").attr("checked", true);
        } else {
            $("#popup-meta-checker-function-switch").attr("checked", false);
        }
    }
    $("#popup-meta-checker-function-switch").on('click', function () {
        clickSwitch(domain);
    });
}

function clickSwitch(domain) {
    let background = chrome.extension.getBackgroundPage();
    if ($("#popup-meta-checker-function-switch").is(':checked')) {
        background.logToES({event_name: 'popupActiveButtonClickOn', text: domain, value: 1});
        background.deleteUserBlackList(domain);
    } else {
        background.logToES({event_name: 'popupActiveButtonClickOff', text: domain, value: 1});
        background.addUserBlackList(domain);
    }
}


// 打开秘塔主页并跳转登录
$('#loginButton').click(e => {
    let background = chrome.extension.getBackgroundPage();
    background.logToES({event_name: 'popupLoginButtonClick', text: '', value: 1});
    let xiezuocatUrl = chrome.extension.getBackgroundPage().getGlobalConfig().xiezuocatUrl;
    window.open(xiezuocatUrl + "/#/?redirect=account");
});



// 在checker中新建一个文档进行编辑
$('#newDocument').click(e => {
    let background = chrome.extension.getBackgroundPage();
    background.logToES({event_name: 'popupNewDocumentButtonClick', text: '', value: 1});
    $('#newDocumentForm').submit();
});

//一键剪裁正文和保存PDF
$('#extractAndSavePDF').click(e => {
    let background = chrome.extension.getBackgroundPage();
    background.logToES({event_name: 'popupExtractButtonClick', text: '', value: 1});
    chrome.tabs.query({active: true, currentWindow: true}, function (tab) {//获取当前tab
        //向tab发送请求
        if (tab[0].url.startsWith("https://chrome.google.com/webstore")) {
            showSpecialTips("该网页无法进行使用。", false);
        } else {
            chrome.tabs.sendMessage(tab[0].id, {cmd: "extract"}, function (response) {
                if (response && response.htmlText) {
                    background.parseHtml(response.htmlText, function (res) {
                        newDoc(res.msg)
                    })
                } else {
                    showSpecialTips("当前页面需要进行刷新。", true)
                }
            });
        }
    });
});

function newDoc(text) {
    let xiezuocatUrl = chrome.extension.getBackgroundPage().getGlobalConfig().xiezuocatAPIUrl;
    let url = xiezuocatUrl + "/html/newBrowserDoc";
    let tempForm = document.createElement("form");
    tempForm.id = "open_in_metasota_checker_form";
    tempForm.method = "POST";
    tempForm.target = "_blank";
    tempForm.action = url;
    tempForm.acceptCharset = 'UTF-8';
    tempForm.style.display = "none";

    //添加参数
    let opt = document.createElement("textarea");
    opt.name = "content";
    opt.value = text;
    tempForm.appendChild(opt);
    document.body.appendChild(tempForm);

    tempForm.submit();
    document.body.removeChild(tempForm);
}

//
$('#ocrButton').click(e => {
    let background = chrome.extension.getBackgroundPage();
    background.logToES({event_name: 'popupOcrButtonClick', text: '', value: 1});
    chrome.tabs.query({active: true, currentWindow: true}, function (tab) {//获取当前tab
        if (tab[0].url.startsWith("https://chrome.google.com/webstore")) {
            showSpecialTips("该网页无法进行使用。", false);
        } else {
            // 向tab发送请求
            chrome.tabs.sendMessage(tab[0].id, {cmd: "ocr"}, function (response) {
                if (response && response.ocrActive) {
                    window.close();
                } else {
                    showSpecialTips("当前页面需要进行刷新。", true);
                }
            });
        }

    });
});


function showSpecialTips(content, showRefreshButton = false) {
    let specialTipsEle = $('#special-tips');
    let specialTipsContentText = $('#popup-special-tip-content-text');
    let specialTipsRefreshButton = $('#popup-special-tip-refresh-button');
    let specialTipsConfirmButton = $('#popup-special-tip-confirm-button');
    specialTipsContentText.text(content);
    if (showRefreshButton) {
        specialTipsConfirmButton.text("取消");
        specialTipsRefreshButton.show();
    } else {
        specialTipsConfirmButton.text("确认");
        specialTipsRefreshButton.hide();
    }
    specialTipsEle.show();
}

function hideSpecialTips() {
    let specialTipsEle = $('#special-tips');
    let specialTipsContentText = $('#popup-special-tip-content-text');
    specialTipsEle.hide();
    specialTipsContentText.text("");
}

$('#popup-special-tip-refresh-button').click(e => {
    chrome.tabs.query({active: true, currentWindow: true}, function (tabs) {
        chrome.tabs.update(tabs[0].id, {url: tabs[0].url});
        hideSpecialTips();
    });
});

$('#popup-special-tip-confirm-button').click(e => {
    hideSpecialTips();
});
